/*-------------------------------------------------------------------------
 *
 * pg_attrdef_d.h
 *    Macro definitions for pg_attrdef
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_ATTRDEF_D_H
#define PG_ATTRDEF_D_H

#define AttrDefaultRelationId 2604
#define AttrDefaultIndexId 2656
#define AttrDefaultOidIndexId 2657

#define Anum_pg_attrdef_oid 1
#define Anum_pg_attrdef_adrelid 2
#define Anum_pg_attrdef_adnum 3
#define Anum_pg_attrdef_adbin 4

#define Natts_pg_attrdef 4


#endif							/* PG_ATTRDEF_D_H */
