/*-------------------------------------------------------------------------
 *
 * pg_publication_rel_d.h
 *    Macro definitions for pg_publication_rel
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_PUBLICATION_REL_D_H
#define PG_PUBLICATION_REL_D_H

#define PublicationRelRelationId 6106
#define PublicationRelObjectIndexId 6112
#define PublicationRelPrrelidPrpubidIndexId 6113
#define PublicationRelPrpubidIndexId 6116

#define Anum_pg_publication_rel_oid 1
#define Anum_pg_publication_rel_prpubid 2
#define Anum_pg_publication_rel_prrelid 3
#define Anum_pg_publication_rel_prqual 4
#define Anum_pg_publication_rel_prattrs 5

#define Natts_pg_publication_rel 5


#endif							/* PG_PUBLICATION_REL_D_H */
