/*-------------------------------------------------------------------------
 *
 * pg_depend_d.h
 *    Macro definitions for pg_depend
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_DEPEND_D_H
#define PG_DEPEND_D_H

#define DependRelationId 2608
#define DependDependerIndexId 2673
#define DependReferenceIndexId 2674

#define Anum_pg_depend_classid 1
#define Anum_pg_depend_objid 2
#define Anum_pg_depend_objsubid 3
#define Anum_pg_depend_refclassid 4
#define Anum_pg_depend_refobjid 5
#define Anum_pg_depend_refobjsubid 6
#define Anum_pg_depend_deptype 7

#define Natts_pg_depend 7


#endif							/* PG_DEPEND_D_H */
