/*-------------------------------------------------------------------------
 *
 * pg_ts_config_map_d.h
 *    Macro definitions for pg_ts_config_map
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_TS_CONFIG_MAP_D_H
#define PG_TS_CONFIG_MAP_D_H

#define TSConfigMapRelationId 3603
#define TSConfigMapIndexId 3609

#define Anum_pg_ts_config_map_mapcfg 1
#define Anum_pg_ts_config_map_maptokentype 2
#define Anum_pg_ts_config_map_mapseqno 3
#define Anum_pg_ts_config_map_mapdict 4

#define Natts_pg_ts_config_map 4


#endif							/* PG_TS_CONFIG_MAP_D_H */
