/*-------------------------------------------------------------------------
 *
 * pg_parameter_acl_d.h
 *    Macro definitions for pg_parameter_acl
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_PARAMETER_ACL_D_H
#define PG_PARAMETER_ACL_D_H

#define ParameterAclRelationId 6243
#define PgParameterAclToastTable 6244
#define PgParameterAclToastIndex 6245
#define ParameterAclParnameIndexId 6246
#define ParameterAclOidIndexId 6247

#define Anum_pg_parameter_acl_oid 1
#define Anum_pg_parameter_acl_parname 2
#define Anum_pg_parameter_acl_paracl 3

#define Natts_pg_parameter_acl 3


#endif							/* PG_PARAMETER_ACL_D_H */
