/*-------------------------------------------------------------------------
 *
 * pg_shdescription_d.h
 *    Macro definitions for pg_shdescription
 *
 * Portions Copyright (c) 1996-2022, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_SHDESCRIPTION_D_H
#define PG_SHDESCRIPTION_D_H

#define SharedDescriptionRelationId 2396
#define PgShdescriptionToastTable 2846
#define PgShdescriptionToastIndex 2847
#define SharedDescriptionObjIndexId 2397

#define Anum_pg_shdescription_objoid 1
#define Anum_pg_shdescription_classoid 2
#define Anum_pg_shdescription_description 3

#define Natts_pg_shdescription 3


#endif							/* PG_SHDESCRIPTION_D_H */
